@@ -3,7 +3,7 @@
  *              support functions for the net-tools.
  *              (NET-3 base distribution).
  *
- * Version:     $Id: econet.c,v 1.6 1999/01/05 20:53:23 philip Exp $
+ * Version:     $Id: econet.c,v 1.8 1999/04/20 13:31:50 philip Exp $
  *
  * Author:      Philip Blundell <philb@gnu.org>
  *
@@ -23,10 +23,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <stdio.h>
-
-#ifndef _NETECONET_EC_H
-#include <linux/if_ec.h>
-#endif
+#include <neteconet/ec.h>
 
 #include "version.h"
 #include "net-support.h"
@@ -82,7 +79,7 @@ struct aftype ec_aftype =
     ec_print, ec_sprint, ec_input, NULL,
     NULL, NULL, NULL,
     -1,
-    NULL
+    "/proc/sys/net/econet"
 };
 
 #endif				/* HAVE_AFECONET */
