@@ -64,6 +64,7 @@ struct hwtype {
     char *(*sprint) (struct sockaddr *);
     int (*input) (char *, struct sockaddr *);
     int (*activate) (int fd);
+    int suppress_null_addr;
 };
 
 
@@ -73,6 +74,7 @@ extern void          print_hwlist(int type);
 extern struct aftype *get_aftype(const char *name);
 extern struct aftype *get_afntype(int type);
 extern void          print_aflist(int type);
+extern int           hw_null_address(struct hwtype *hw, void *addr);
 
 extern int getargs(char *string, char *arguments[]);
 
